import { CacheResolutionMethod, KeySerializationMethod, SimpleCache } from './withSimpleCaching';
/**
 * exposes the cache-wrapped method along with some primitives which enable extending the caching logic
 *
 * specifically
 * - exposes a way to `invalidate` the cache, for a given input (e.g., to support external triggers for invalidation)
 * - exposes a way to `update` the cache, for a given input (e.g., to support write-through caching and optimistic caching)
 *
 * relevance
 * - when wrapping logic to cache the user is able to specify several caching options (e.g., key serialization method, value serialization method, etc)
 * - in order to define their own `invalidation` and `update` methods, without this function, the user would need to access these caching options per function elsewhere
 * - this function makes it easy to utilize and extend cache invalidation + update commands for the wrapped logic, by managing the references to the caching options on behalf of the user
 */
export declare const withExtendableCaching: <LR extends unknown, CR extends unknown, L extends (...args: any[]) => LR>(logic: L, options: {
    cache: SimpleCache<CR> | CacheResolutionMethod<L, CR>;
    serialize?: {
        key?: KeySerializationMethod<Parameters<L>> | undefined;
        value?: ((returned: LR) => CR) | undefined;
    } | undefined;
    deserialize?: {
        value?: ((cached: CR) => LR) | undefined;
    } | undefined;
    secondsUntilExpiration?: number | undefined;
}) => {
    execute: L;
    invalidate: ({ forInput, }: {
        /**
         * invalidate the cache for this input
         */
        forInput: Parameters<L>;
    }) => Promise<void>;
    update: ({ forInput, toValue, }: {
        /**
         * update the cache for this input
         */
        forInput: Parameters<L>;
        /**
         * update the cache to this value
         */
        toValue: ReturnType<L>;
    }) => Promise<void>;
};
