export interface SimpleCache<T> {
    get: (key: string) => T | undefined | Promise<Awaited<T> | undefined>;
    set: (key: string, value: T, options?: {
        secondsUntilExpiration?: number;
    }) => void | Promise<void>;
}
export declare type KeySerializationMethod<P> = (args: P) => string;
export declare const noOp: <LR, CR>(value: LR) => CR;
export declare const defaultKeySerializationMethod: {
    (value: any, replacer?: ((this: any, key: string, value: any) => any) | undefined, space?: string | number | undefined): string;
    (value: any, replacer?: (string | number)[] | null | undefined, space?: string | number | undefined): string;
};
export declare const defaultValueSerializationMethod: <LR, CR>(value: LR) => CR;
export declare type CacheResolutionMethod<L extends (...args: any[]) => any, CR extends any> = (...args: Parameters<L>) => SimpleCache<CR>;
export declare const getCacheFromCacheOption: <L extends (...args: any[]) => any, CR extends unknown>({ forInput, cacheOption, }: {
    forInput: Parameters<L>;
    cacheOption: SimpleCache<CR> | CacheResolutionMethod<L, CR>;
}) => SimpleCache<CR>;
/**
 * caches the promise of each invocation, based on serialization of the inputs.
 *
 * by default, this uses the `simple-in-memory-cache`, but you can pass in a `get` and `set` method for any persistent storage (e.g., local storage, cookie storage, dynamodb, etc)
 *
 * for example:
 * ```ts
 * const getApiResult = withSimpleCaching(({ name, number }) => axios.get(URL, { name, number }));
 * const result1 = getApiResult({ name: 'casey', number: 821 }); // calls the api, puts promise of results into cache, returns that promise
 * const result2 = getApiResult({ name: 'casey', number: 821 }); // returns the same promise from above, because it was found in cache - since same input as request above was used
 * expect(result1).toBe(result2); // same exact object - the promise
 * expect(await result1).toBe(await result2); // same exact object - the result of the promise
 * ```
 */
export declare const withSimpleCaching: <LR extends unknown, CR extends unknown, L extends (...args: any[]) => LR>(logic: L, { cache: cacheOption, serialize: { key: serializeKey, value: serializeValue, }, deserialize: { value: deserializeValue }, secondsUntilExpiration, }: {
    cache: SimpleCache<CR> | CacheResolutionMethod<L, CR>;
    serialize?: {
        key?: KeySerializationMethod<Parameters<L>> | undefined;
        value?: ((returned: LR) => CR) | undefined;
    } | undefined;
    deserialize?: {
        value?: ((cached: CR) => LR) | undefined;
    } | undefined;
    secondsUntilExpiration?: number | undefined;
}) => L;
